*DECK THMH2O
      SUBROUTINE THMH2O(I,J,K,PINLET,MFLOW,EPS,ENT,PHI,HD,IFLUID,IHCONV,
     > KHCONV,TL,RHO,RHOL,HCONV,TG,X)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solution of the convection equation along a single channel.
*
*Copyright:
* Copyright (C) 2012 Ecole Polytechnique de Montreal.
*
*Author(s): A. Hebert, P. Gallet
*
*Parameters: input
* I       position of channel alon X-axis
* J       position of channel alon Y-axis
* K       position of channel alon Z-axis
* PINLET  pressure in Pascal
* MFLOW   massic coolant flow rate in Kg/m^2/s
* EPS     coolant void fraction
* ENT     four values of enthalpy in J/Kg to be used in Gaussian
*         integration
* PHI     heat flow exchanged between clad and fluid in W/m^2
* HD      hydraulic diameter in m
* IFLUID  type of fluid (0=H2O; 1=D2O).
* IHCONV  flag indicating HCONV chosen (0=default/1=user-provided).
* KHCONV  fixed user-provided HCONV value in W/m^2/K.
*
*Parameters: output
* TL      fluid temperature in K
* RHO     fluid density in Kg/m^3
* RHOL    liquid density in kg/m^3
* HCONV   heat transfer coefficient between clad and fluid in W/m^2/K
* TG      clad temperature in K
* X       steam thermodynamic quality
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER I,J,K,IFLUID,IHCONV
      REAL PINLET,MFLOW,EPS,ENT(4),TL,PHI,RHO,RHOL,HCONV,HD,KHCONV,TG,X
*----
*  LOCAL VARIABLES
*----
      REAL HGSAT,XMULS,W(4),CPL,XLL,TL1,TL2,TL3,TL4,RHOL1,RHOL2,RHOL3,
     > RHOL4,HL(4),REL,PRL,HLSAT,R1,R2,R3,R4,R5,R11,TSAT,RHOLSAT,RHOG,
     > HMAVG,TB,TEMP
      CHARACTER HSMG*131
*
      DATA W /0.347855,0.652145,0.652145,0.347855/
*----
*  COMPUTE THE DENSITY OF THE SATURATED STEAM
*----
      IF(IFLUID.EQ.0) THEN
         CALL THMSAT(PINLET,TSAT)
         CALL THMTX(TSAT,0.0,RHOLSAT,HLSAT,R3,R4,R5)
         CALL THMTX(TSAT,1.0,RHOG,HGSAT,R3,R4,R5)
      ELSE IF(IFLUID.EQ.1) THEN
         CALL THMHST(PINLET,TSAT)
         CALL THMHTX(TSAT,0.0,RHOLSAT,HLSAT,R3,R4,R5)
         CALL THMHTX(TSAT,1.0,RHOG,HGSAT,R3,R4,R5)
      ENDIF
*----
*  COMPUTE THE DENSITY AND TEMPERATURE OF THE LIQUID
*----
      HMAVG=0.5*(ENT(2)+ENT(3))
      HL(1)=MIN1(ENT(1),HLSAT)
      HL(2)=MIN1(ENT(2),HLSAT)
      HL(3)=MIN1(ENT(3),HLSAT)
      HL(4)=MIN1(ENT(4),HLSAT)
      CALL THMPH(IFLUID,PINLET,HL(1),R11,TL1)
      CALL THMPH(IFLUID,PINLET,HL(2),R11,TL2)
      CALL THMPH(IFLUID,PINLET,HL(3),R11,TL3)
      CALL THMPH(IFLUID,PINLET,HL(4),R11,TL4)
      IF(IFLUID.EQ.0) THEN
        CALL THMPT(PINLET,TL1,RHOL1,R2,R3,R4,R5)
        CALL THMPT(PINLET,TL2,RHOL2,R2,R3,R4,R5)
        CALL THMPT(PINLET,TL3,RHOL3,R2,R3,R4,R5)
        CALL THMPT(PINLET,TL4,RHOL4,R2,R3,R4,R5)
        IF(ABS(TSAT-TL1).LT.0.1) CALL THMTX(TSAT,0.0,RHOL1,R2,R3,R4,R5)
        IF(ABS(TSAT-TL2).LT.0.1) CALL THMTX(TSAT,0.0,RHOL2,R2,R3,R4,R5)
        IF(ABS(TSAT-TL3).LT.0.1) CALL THMTX(TSAT,0.0,RHOL3,R2,R3,R4,R5)
        IF(ABS(TSAT-TL4).LT.0.1) CALL THMTX(TSAT,0.0,RHOL4,R2,R3,R4,R5)
      ELSE IF(IFLUID.EQ.1) THEN
        CALL THMHPT(PINLET,TL1,RHOL1,R2,R3,R4,R5)
        CALL THMHPT(PINLET,TL2,RHOL2,R2,R3,R4,R5)
        CALL THMHPT(PINLET,TL3,RHOL3,R2,R3,R4,R5)
        CALL THMHPT(PINLET,TL4,RHOL4,R2,R3,R4,R5)
        IF(ABS(TSAT-TL1).LT.0.1) CALL THMHTX(TSAT,0.0,RHOL1,R2,R3,R4,R5)
        IF(ABS(TSAT-TL2).LT.0.1) CALL THMHTX(TSAT,0.0,RHOL2,R2,R3,R4,R5)
        IF(ABS(TSAT-TL3).LT.0.1) CALL THMHTX(TSAT,0.0,RHOL3,R2,R3,R4,R5)
        IF(ABS(TSAT-TL4).LT.0.1) CALL THMHTX(TSAT,0.0,RHOL4,R2,R3,R4,R5)
      ENDIF
      TL=0.5*(W(1)*TL1+W(2)*TL2+W(3)*TL3+W(4)*TL4)
      RHOL=0.5*(W(1)*RHOL1+W(2)*RHOL2+W(3)*RHOL3+W(4)*RHOL4)
      TB=TSAT-0.1
      IF(TL.LT.TB) THEN
         TEMP=TL
      ELSE
         TEMP=TB
      ENDIF
      IF(IFLUID.EQ.0) THEN
         CALL THMPT(PINLET,TEMP,R1,R2,XLL,XMULS,CPL)
      ELSE IF(IFLUID.EQ.1) THEN
         CALL THMHPT(PINLET,TEMP,R1,R2,XLL,XMULS,CPL)
      ENDIF
*----
*  COMPUTE THE FLUID DENSITY
*----
      IF(HMAVG.LE.HLSAT) THEN
*        One phase liquid
         X=0.0
         IF(EPS.EQ.0.0) THEN
           RHO=RHOL
         ELSE
           RHO=EPS*RHOG+(1.0-EPS)*RHOLSAT
         ENDIF
      ELSE IF(HMAVG.LT.HGSAT) THEN
*        Two-phase flow
         X=(HMAVG-HLSAT)/(HGSAT-HLSAT)
         IF(EPS.EQ.0.0) THEN
           RHO=1.0/(X/RHOG+(1.0-X)/RHOL)
         ELSE
           RHO=EPS*RHOG+(1.0-EPS)*RHOLSAT
         ENDIF
      ELSE
*        Saturated steam
         X=1.0
         RHO=RHOG
      ENDIF
*
      IF(IHCONV.EQ.0) THEN
*----
*  THERMAL EXCHANGE BETWEEN CLAD AND FLUID USING THE DITTUS AND BOELTER
*  CORRELATION
*  REL: Reynolds number of liquid phase
*  PRL: Prandtl number of liquid phase
*----
         IF(X.GE.0.0) THEN
            REL=MFLOW*(1.0-X)*HD/XMULS
         ELSE
            REL=MFLOW*HD/XMULS
         ENDIF
         PRL=XMULS*CPL/XLL
         HCONV=0.023*XLL/HD*REL**0.8*PRL**0.4
      ELSE IF(IHCONV.EQ.1) THEN
         HCONV=KHCONV
      ENDIF
      IF(HCONV.GT.0.0) THEN
         TG=TL+PHI/HCONV
      ELSE
         WRITE(HSMG,'(34HTHMH2O: DRY OUT REACHED IN CHANNEL,3I5)') I,J,K
         CALL XABORT(HSMG)
      ENDIF
      RETURN
      END
